﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using WebConfigEncrypt.Properties;

namespace WebConfigEncrypt {
    partial class ConfigureServer : Form {
        
        
        public ConfigureServer() {
            InitializeComponent();

            this.AcceptButton = this.ConfirmChanges;
            this.CancelButton = this.CancelChanges;
            
        }

        private void CancelChanges_Click(object sender, EventArgs e) {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void ConfirmChanges_Click(object sender, EventArgs e) {

            //make sure this is okay
            if (string.IsNullOrEmpty(this.Server.Text.Trim())) {
                this.Server.Text = "localhost";
            }

            //verify the connection
            bool local = Regex.IsMatch(this.Server.Text, Resources.Regex_MatchLocalHost, RegexOptions.IgnoreCase);

            //make sure the strings are there
            this.Username.Text = (this.Username.Text ?? string.Empty).Trim();
            this.Domain.Text = (this.Domain.Text ?? string.Empty).Trim();
            this.Server.Text = (this.Server.Text ?? string.Empty).Trim();

            //if this isn't a local connection, make sure the username and password are enterd
            bool credentials = !(
                string.IsNullOrEmpty(this.Username.Text) ||
                string.IsNullOrEmpty(this.Domain.Text) ||
                string.IsNullOrEmpty(this.Password.Text) ||
                string.IsNullOrEmpty(this.ConfirmPassword.Text)
                );

            //check the name of the server
            if (local && !credentials) {
                DialogResult confirm = MessageBox.Show(
                    "You're using a local connection, WMI does not use credentials for local connections.\n\nDo you want to proceed?",
                    "Nkript - Unused Credentials",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Information
                    );

                //if canceling
                if (confirm != DialogResult.Yes) { return; }

            }

            //check if this isn't enough information
            if (!local && !credentials) {
                MessageBox.Show(
                    "In order to connect to the server provided, you must supply all of the required credentials",
                    "Nkript - Missing Credentials",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information
                    );
                return;
            }

            //make sure the passwords are okay
            if (!this.Password.Text.Equals(this.ConfirmPassword.Text)) {
                MessageBox.Show(
                    "The passwords entered do not match.",
                    "Nkript - Password Problem",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information
                    );
                return;
            }

            //otherwise, make sure everything is nice and formatted

            //return the success
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

    }
}
