﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WebConfigEncrypt.Types;
using System.Xml.Linq;
using WebConfigEncrypt.Interfaces;
using System.Threading;
using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace WebConfigEncrypt {

    public partial class EncryptSection : Form {

        #region Constructors

        //creates the form to handle encrypting the config
        public EncryptSection(XDocument document, WebConfigInformation info, WebServerInterface server) {
            InitializeComponent();

            this._Document = document;
            this._Config = info;
            this._Interface = server;
            
            //update the display for the first time
            this.Load += (s, e) => {

                //display the contents of this document
                this.XmlOutput.Text = this._Document.ToString();

                //add an entry for each root section - not sure how this works for later
                this.AvailableSections.Items.Clear();
                foreach (XElement section in this._Document.Root.Elements()) {
                    this.AvailableSections.Items.Add(section.Name);
                }

            };

        }

        #endregion

        #region Properties and Fields

        private WebConfigInformation _Config;
        private XDocument _Document;
        private WebServerInterface _Interface;

        #endregion

        #region Button Clicks

        //encrypts a selected section
        private void BeginEncrypt_Click(object sender, EventArgs e) {
            this._HandleChange(true);
        }

        //decrypts a selected section
        private void BeginDecrypt_Click(object sender, EventArgs e) {
            this._HandleChange(false);
        }

        #endregion

        #region Updating The Config

        //changes a web.config based on a selection
        private void _HandleChange(bool encrypt) {

            //make sure the selection was made
            if (this.AvailableSections.SelectedItem == null || string.IsNullOrEmpty((this.AvailableSections.SelectedItem.ToString() ?? string.Empty).Trim())) {
                MessageBox.Show(
                    "You need to select which section you plan to change",
                    "Nkript - Select a section",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Question
                    );
                return;
            }

            //get the selection
            string selection = this.AvailableSections.SelectedItem.ToString();

            //create some work for a thread
            Action work = () => {

                //create the share to work with
                string target = this._Interface.CreateTemporaryShare(this._Config.Path);
                DateTime lastUpdate = File.GetLastWriteTime(target);

                //send the update request
                if (encrypt) {
                    this._Interface.EncryptSection(this._Config, selection);
                }
                else {
                    this._Interface.DecryptSection(this._Config, selection);
                }

                //give a moment for work
                Thread.Sleep(1000);

                //keep watching until the file has been updated
                int safety = 60;
                bool loaded = false;
                while (safety-- > 0) {

                    //if it appears available, try and read it
                    try {

                        //check for the file
                        if (File.Exists(target) && File.GetLastWriteTime(target) > lastUpdate) {
                            XDocument updated = this._Interface.ReadRemoteFile(target);

                            //cheesy way of evaluating this, but can't figure out the best
                            //way to guarantee this really did change
                            if (this._Document.ToString().Length == updated.ToString().Length) { continue; }
                            this._Document = updated;
                            loaded = true;
                            break;
                        }
                    }
                    catch {

                        //there is no reliable way to do this
                    }
                    Thread.Sleep(500);
                }

                //close our temporary directory
                try {
                    this._Interface.RemoveTemporaryShare();
                }
                catch {

                    //just notify they may need to clean it up themselves
                    MessageBox.Show(
                        "Unable to clear the temporary share to access this web.config file. You may need to remove this yourself.",
                        "Nkript - Could not remove share",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning
                        );
                }

                //if nothing changed, notify them
                if (!loaded) {
                    MessageBox.Show(
                        "No changes were detected after running your requested command. This could mean that the file was unavailable to be read or that the request failed.",
                        "Nkript - No changes detected",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning
                        );
                }

                //update the UI
                Action update = () => {
                    this.XmlOutput.Text = this._Document.ToString();
                    this.Loader.Visible = false;
                    this.Status.Text = "Waiting...";

                    //notify this finished
                    MessageBox.Show(
                        "The requested section was updated.",
                        "Nkript - Section updated",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information
                        );
                };
                this.Invoke(update);

            };

            //start the work request
            this.Loader.Visible = true;
            this.Status.Visible = true;
            this.Status.Text = "Updating web.config file...";
            work.BeginInvoke(null, null);

        }

        #endregion

        #region Help Information

        private void MoreInfo_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
            Process.Start("http://msdn.microsoft.com/en-us/library/ms998280.aspx");
        }

        #endregion

    }

}
